@@ -4,10 +4,17 @@
 htop - OpenBSDProcess.h
 (C) 2015 Hisham H. Muhammad
 (C) 2015 Michael McConville
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include <stdbool.h>
+
+#include "Object.h"
+#include "Process.h"
+#include "Settings.h"
+
+
 typedef enum OpenBSDProcessFields_ {
    // Add platform-specific fields here, with ids >= 100
    LAST_PROCESSFIELD = 100,
@@ -21,20 +28,16 @@ typedef struct OpenBSDProcess_ {
 
 #define Process_isUserlandThread(_process) (_process->pid != _process->tgid)
 
-extern ProcessClass OpenBSDProcess_class;
+extern const ProcessClass OpenBSDProcess_class;
 
 extern ProcessFieldData Process_fields[];
 
 extern ProcessPidColumn Process_pidColumns[];
 
-OpenBSDProcess* OpenBSDProcess_new(Settings* settings);
+Process* OpenBSDProcess_new(const Settings* settings);
 
 void Process_delete(Object* cast);
 
-void OpenBSDProcess_writeField(Process* this, RichString* str, ProcessField field);
-
-long OpenBSDProcess_compare(const void* v1, const void* v2);
-
-bool Process_isThread(Process* this);
+bool Process_isThread(const Process* this);
 
 #endif
